/*==============================================================================


   Indicador de reserva de combustivel (para uso veicular)

   Autor: Cludio Lrios

   Data: 17/09/2014

   Pic 12F675

   Caracteristicas:
   1) Pode ser usado com boia invertida ou normal.
    (invertida quando sua resistencia  baixa com tanque cheio. A bia 'normal'
     quando sua resistncia  alta com tanque cheio). Use sem jumper para boia
    invertida (jumper no pino 6).
   2) Pode ser adicionado ou retirado um delay no acionamento. Com jumper no
    pino 7 ser sem delay, e com jumper, com delay.
   3) O momento do acendimento  programado usando um boto 'learn'. Basta
    deixar ficar no inicio da reserva e apertar 'learn' para gravar este ponto.
   4) Indicao luminosa mediante led e sonora, usando um disco piezoeltrico.
    No inicio do acionamento, ouve-se 4 bips e aps um tempo, haver  repetio
    destes bips.
    5) Indicao luminoso poder ser piscante ou direta. Use o jump no pino 2 
    para a escolha.
    
    Jumpers:
    pino 2 -> colocado= led acende direto na reserva / sem jumper= pisca led
    pino 7 -> colocado= sem delay no acionamento   / sem jumper= com delay
    pino 6 -> colocado= boia normal  / sem jumper= boia invertida
    
    
    Pinagem do pic 12f675:
    
    1- vcc (+5v)
    2- saida para led indicador de reserva via resitor de 470 ohms
       Tambm do pino, via resistor de 470 ohms p/ jumper ao gnd (modo no_pisca)
    3- saida para o led 'learn' via resistor de 470 ohms
       Tambm do pino, via resistor de 470 ohms ao piezo eletrico para o gnd
    4- boto learn ao gnd com pullup de 10k ao vcc.
    5- entrada da boia, sendo do pino a um resistor de 10k ao gnd e do pino 
       a um resistor de 15k a boia (divisor de tenso de entrada).
    6- jumper ao gnd (modo invertido)
    7- jumper ao gnd (modo no_delay)
    8- gnd

   Este arquivo  parte integrante do Blog: Larios.Tecnologia.Ws
   Uso didtico (experimental).

==============================================================================*/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#zero_ram
#byte  tris     = 0x85
#byte  gpio     = 0x05
#byte  wpu      = 0x95
#bit   tris_res = 0x85.5
#bit   led_res  = 0x05.5  // led indicador de reserva
#bit   led_prog = 0x05.4  //led indicador de programao
#bit   sw1      = 0x05.3  //interruptor de programao
#bit   j_nodel  = 0x05.0  //jump para delay no acionamento
#bit   j_inv    = 0x05.1  //jump para inverter

const int8  num_bips = 4; //numero de bips ao indicar reserva
const int8  v_cnt =10;//atraso 3 seg. aps ficar na reserva (agitao do tanque)
const int8 prox_bip = 250; // numero de piscada at o proximo sinal sonoro

int8 valor,cnt,cont1,cont2;
int1 flag_res,flag_lim,flag_bip,flag_nop;




void mdelay(){//delay entre leituras analogica/digital com gerao de bip
  int8 a;
 
   if(++cont2==prox_bip) {flag_bip=0; cont2=0;cont1=0;}//conta tempo p/ novo bip
   if((led_res)&&(!flag_bip)) {
   
     for(a=0;a<250;a++){//gera bip
     led_prog=1;//no pino do led prog
     delay_us(250);
     led_prog=0;
     delay_us(250);
     
     }
     if(++cont1==num_bips){flag_bip=1;cont2=0;cont1=0;}//bipa novamente
     if(!flag_nop){ delay_ms(300);}//acrescenta delay se led em modo 'no_pisca'
   }
   
   else{ delay_ms(300);}


}


/*==============================================================================
                           INICIO DO PROGRAMA
==============================================================================*/
void main() {

   setup_adc_ports(sAN2|VSS_VDD);//liga conversor analgico/digital
   setup_adc(ADC_CLOCK_DIV_8); //ajusta a frequncia de operao
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1); //timer 0
   setup_timer_1(T1_DISABLED); //desabilita timer1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga Vref
   PORT_A_PULLUPS(true);//liga geral dos pullups
   tris=0b001111;//ajusta trisio
   wpu=0xff;//liga pullup individual
   gpio=0;//zera gpio
   set_adc_channel(2);//escolhe canal do conversor analgico/digital para 2
   delay_us(10);//delay
 
/*==============================================================================
                           LOOP  PRINCIPAL
==============================================================================*/

   while(1){
     tris_res=1;//entrada no jump de led no_pisca
    if(led_res){flag_nop=1;}//testa se led dever piscar ou direto na reserva
    else {flag_nop=0;}
    tris_res=0; //saida

    valor = read_adc();//le valor da tenso na 'boia' do tanque

    if(!sw1){ //boto 'learn' apertado? - sim, programa valor na Eeprom
      write_eeprom(0,valor);//grava na EEprom
      led_prog=1; //acende led 'prog'
      delay_ms(500);//delay de 0,5 segundos com led aceso
      led_prog=0;//apaga led 'prog'
      while(!sw1);//aguarda soltar o boto
      }

     if(!j_inv){
   //jump 'inverter' colocado? Sim. Tenso da boia 'menor' que EEprom acende led
         if(valor<read_eeprom(0)) { flag_lim=1;} //seta flag
         else { flag_lim=0;}//reseta flag

      }
      else{
   //No. Tenso da boia 'maior' que EEprom  acende led

       if(valor>read_eeprom(0)) { flag_lim=1;} //seta flag
         else { flag_lim=0;} //reseta flag

      }


      if(flag_lim) {//testa flag
   //Jump 'no_delay' colocado?
         if(!j_nodel){flag_res=1;}//Sim. Sem delay
         else{
         if(++cnt==v_cnt){ flag_res=1; cnt--;}//No. Com delay
         }
      }
       else {
        //Jump 'no_delay' colocado?
          if(!j_nodel){flag_res=0;}//Sim. Sem delay
          else{
         if(--cnt==0xff){ flag_res=0; cnt++;}//No. Com delay
          }
       }

    if(flag_res) { //testa de deve acionar aviso de reserva no tanque
      if(flag_nop){ led_res=!led_res; } //pisca led de reserva
      else {led_res=1;}
     }
     else {led_res=0; cont1=0; flag_bip=0;} //mantm apagado
    mdelay(); //tempo entre as  piscadas

   }//while
}//main
